      subroutine tkeprofile(ps,ttlev,ulev,vlev,tkelev,nuvz,
C                           i    i    i    i      i    i
     &   sigmahalf,ptop,p00,ts0,ta,hmix,hmixplus)
C            i      i    i   i   i   o      o
************************************************************************
* $Id: tkeprofile.f,v 1.1 2006/08/09 14:15:55 paul Exp paul $
* $Log: tkeprofile.f,v $
* Revision 1.1  2006/08/09 14:15:55  paul
* Initial revision
*
*                                                                      *
*                FLEXPART (MM5) SUBROUTINE TKEPROFILE                  *
*                          Author: G. WOTAWA                           *
*                              1998-08-06                              *
*                                                                      *
************************************************************************
*                                                                      *
* SHORT DESCRIPTION:                                                   *
*                                                                      *
* This subroutine calculates the mixing height from the vertical       *
* profile of turbulent kinetic energy (TKE). Afterwards, the possible  *
* excess mixing height needed for the subgridscale terrain variability *
* parameterisation is calculated from the Brunt-Vaisala frequency     *
* at z=hmix.                                                           *
*                                                                      *
************************************************************************
*                                                                      *
* ALGORITHM OF MIXING HEIGHT COMPUTATION:                              *
*                                                                      *
* 3 CASES:                                                             *
*                                                                      *
* CASE 1: WEAK TURBULENCE CASE                                         *
*         TKEMAX < 0.04 M2/S2                                          *
*         HMIX IS SET TO THE HEIGHT OF THE FIRST FULL SIGMA LEVEL      *
*                                                                      *
*                                                                      *
* CASE 2: MODERATE OR STRONG TURBULENCE CASE                           *
*         HMIX IS SET TO THE HEIGHT WHERE TKE FALLS BELOW A CERTAIN    *
*         THRESHOLD VALUE                                              *
*                                                                      *
*   2A: MODERATE TURBULENCE CASE                                       *
*       TKEMAX <= 0.2 M2/S2                                            *
*       THRESHOLD=0.5*TKEMAX                                           *
*                                                                      *
*   2B: STRONG TURBULENCE CASE                                         *
*       TKE > 0.2 M2/S2                                                *
*       THRESHOLD=0.1 M2/S2                                            *
*                                                                      *
************************************************************************
*                                                                      *
* DESCRIPTION OF INPUT:                                                *
*                                                                      *
* PS              (REFERENCE) SURFACE PRESSURE (PA)                    *
* TTLEV           TEMPERATURE PROFILE ON HALF SIGMA LEVELS (K)         *
* ULEV            U WIND SPEED PROFILE ON HALF SIGMA LEVELS (M/S)      *
* VLEV            V WIND SPEED PROFILE ON HALF SIGMA LEVELS (M/S)      *
* TKELEV          TKE PROFILE ON HALF SIGMA LEVELS (M2/S2)             *
* NUVZ            NUMBER OF HALF SIGMA LEVELS                          *
* SIGMAHALF       HALF SIGMA LEVELS (DIMENSIONLESS)                    *
* PTOP            MM5 TOP PRESSURE (PA)                                *
* P00             REFERENCE SEA LEVEL PRESSURE (PA)                    *
* TS0             REFEENCE SEA LEVEL TEMPERATURE (K)                   *
* TA              REFERENCE TEMPERATURE LAPSE RATE (K)                 *
*                                                                      *
* DESCRIPTION OF OUTPUT:                                               *
*                                                                      *
* HMIX            MIXING HEIGHT (M)                                    *
* HMIXPLUS        EXCESS MIXING HEIGHT (M)                             *
*                                                                      *
************************************************************************
      include 'includepar'

      integer nuvz,k,nuvzmix
      real ps,ttlev(nuvz),ulev(nuvz),vlev(nuvz)
      real tkelev(nuvz),sigmahalf(nuvz),ptop,p00,ts0,ta
      real hmix,hmixplus,sigma1,z1,z2,rlin,sigmatoh,geopot1
      real tkemax,tkeweak,tkestrong,tkefract,tkethreshold
      parameter(tkeweak=0.04,tkestrong=0.20,tkefract=0.5)
      parameter(tkethreshold=0.1)
      real tkehmix,ul,vl,p1,p2,theta1,theta2,thetam,bvfsq,bvf,wspeed

      geopot1=sigmatoh(sigmahalf(1),ps,ptop,p00,ts0,ta)
      tkemax=0.0
      do 10 k=1,nuvz
!!		print*,'tkelev',k,tkelev(k)
        if(tkelev(k).le.0.01) goto 15 ! PS: why jump out of the loop???
10      if(tkelev(k).gt.tkemax) tkemax=tkelev(k)

15    nuvzmix=k
	print*, 'hmix1 ', nuvzmix,tkelev(k)
      if(tkemax.lt.tkeweak) then
	

* WEAK TURBULENCE CASE

        sigma1=0.5*(sigmahalf(1)+sigmahalf(2)) ! FIRST FULL SIGMA LEVEL
        hmix=sigmatoh(sigma1,ps,ptop,p00,ts0,ta)-geopot1

* Compute hmixplus from stability between the lowermost two sigma half levels

        z1=sigmatoh(sigmahalf(2),ps,ptop,p00,ts0,ta)-geopot1
        z2=sigmatoh(sigmahalf(3),ps,ptop,p00,ts0,ta)-geopot1
        ul=0.5*(ulev(2)+ulev(3))
        vl=0.5*(vlev(2)+vlev(3))
        wspeed=sqrt(ul**2+vl**2)
        p1=sigmahalf(2)*(ps-ptop)+ptop
        p2=sigmahalf(3)*(ps-ptop)+ptop
        theta1=ttlev(2)*(100000./p1)**kappa
        theta2=ttlev(3)*(100000./p2)**kappa
        thetam=0.5*(theta1+theta2)
        bvfsq=(ga/thetam)*(theta2-theta1)/(z2-z1)
        if(bvfsq.le.0.) then
          hmixplus=4000.             ! Rising not confined by stability
        else
          bvf=sqrt(bvfsq)
          hmixplus=wspeed/bvf*convke ! Rising confined by stability
        endif

      else

* MODERATE/STRONG TURBULENCE: HMIX IS THE HEIGHT WHERE TKE
* FALLS BELOW A CERTAIN THRESHOLD

        if(tkemax.le.tkestrong) then
          tkehmix=tkefract*tkemax
        else
          tkehmix=tkethreshold
        endif

        do 20 k=2,nuvzmix
20        if(tkelev(k).lt.tkehmix) goto 30

30      continue


* Mixing height is situated between levels k-1,k
* -> interpolate linearly

        z1=sigmatoh(sigmahalf(k-1),ps,ptop,p00,ts0,ta)-geopot1
        z2=sigmatoh(sigmahalf(k)  ,ps,ptop,p00,ts0,ta)-geopot1
        if(tkelev(k).ne.tkelev(k-1)) then
          rlin=(tkehmix-tkelev(k-1))/(tkelev(k)-tkelev(k-1))
          hmix=z1+(z2-z1)*rlin
        else
          rlin=0.5
          hmix=0.5*(z1+z2)
        endif

* Compute hmixplus

        ul=ulev(k-1)+(ulev(k)-ulev(k-1))*rlin
        vl=vlev(k-1)+(vlev(k)-vlev(k-1))*rlin
        wspeed=sqrt(ul**2+vl**2)
        p1=sigmahalf(k-1)*(ps-ptop)+ptop
        p2=sigmahalf(k)  *(ps-ptop)+ptop
        theta1=ttlev(k-1)*(100000./p1)**kappa
        theta2=ttlev(k)  *(100000./p2)**kappa
        thetam=0.5*(theta1+theta2)
        bvfsq=(ga/thetam)*(theta2-theta1)/(z2-z1)
        if(bvfsq.le.0.) then
          hmixplus=4000.             ! Rising not confined by stability
        else
          bvf=sqrt(bvfsq)
          hmixplus=wspeed/bvf*convke ! Rising confined by stability
        endif
      endif

	print*,'hmix=',hmix,hmixplus

      return
      end
